#!/usr/bin/perl

use warnings;
use diagnostics;
use strict;
use vars qw/ $opt_h $opt_f $opt_l $opt_r $opt_s $opt_c $opt_v $opt_m $opt_V /;
use Getopt::Std;
use DBI;
use Date::Manip;

	my $Version = "0.41";
	my $host_count = "10";
	my $virus_count = "1";
	my $mail_server = 'localhost';
	my $mail_user = "webadmin\@hillsboroughcounty.org";
	my $log_file = '/var/log/maillog';
	my $user = "clamstat";
	my $pw = "1clamstat2";
	my $database = "clamstat";
	my $serverName = "localhost";
	my $serverPort = "3306";

my $pbh = DBI->connect("DBI:mysql:$database;host=$serverName;port=$serverPort",$user,$pw);
	die "Unable to connect: ".$DBI::errstr."\n" unless (defined $pbh);

my $c1 = DateCalc("today","- 1 days");
my $combinedtime1 = UnixDate("$c1",'%Y-%m-%d %H:%M:%S');
my $combinedtime2 = UnixDate("today",'%Y-%m-%d %H:%M:%S');

my $results=$pbh->prepare(q{SELECT * from logs WHERE time BETWEEN ? AND ? }) or
	die "Unable to prepare our query: ".$pbh->errstr."\n";
$results->execute($combinedtime1,$combinedtime2) or die "Unable to execute our query: ".$pbh->errstr."\n";

getopts( 'h:sc:v:rmV' );	

if ($opt_h) {
	print "Options: \n";
	print "-h Help \n";
	print "-r Show recipients \n";
	print "-s Show senders - Milter only \n";
	print "-c Minimum virus count for unique hosts \n";
	print "-v Minimum virus type count \n";
	print "-m Email report to predefined values set in this perl script  \n";
	print "-V Version \n";
	exit 0
}


my $show_senders;
if ($opt_s) {
	$show_senders = "1";
}

$host_count = $opt_c if ($opt_c);

$virus_count = $opt_v if ($opt_v);

my $show_recipients = "1" if ($opt_r);

my $send_email = "1" if ($opt_m);

if ($opt_V) {
	print "Version $Version \n";
	exit 0
}

	our ($ip_addr, $sender);
	my ($data, $email, %ip_addr, $virus, %virus, $Virus, $Sender, $recipient, $Recipient, $message_id, @text_body, @count, $datetime);


while ($data = $results->fetchrow_arrayref()) {
	$datetime = "$data->[1]";
	$virus = "$data->[2]";
	$ip_addr = "$data->[5]";
	$recipient = "$data->[4]";
	$sender = "$data->[3]";

	#Counts total number of times a virus was sent
	$ip_addr{$virus}++;
	#Counts total number of virus's sent by IP address
	$virus{$ip_addr}++;
	#Counts total number of unique virus's per IP address
	$email->{$ip_addr}{virus}{$virus}++;
	#Counts total number of unique senders per IP address
	if ($show_senders) {
		$email->{$ip_addr}{sender}{$sender}++;
	}
	#Counts total number of unique recipients per IP address
	if ($show_recipients) {
		$email->{$ip_addr}{recipient}{$recipient}++;
	}
}

sub hashValueDescendingVirus {
	   $email->{$ip_addr}{virus}{$b} <=> $email->{$ip_addr}{virus}{$a};
}
		
sub hashValueDescendingRecipient {
	   $email->{$ip_addr}{recipient}{$b} <=> $email->{$ip_addr}{recipient}{$a};
}

sub hashValueDescendingSender {
	   $email->{$ip_addr}{sender}{$b} <=> $email->{$ip_addr}{sender}{$a};
}

sub hashValueDescendingNum {
   $ip_addr{$b} <=> $ip_addr{$a};
}

sub hashValueDescendingIp {
   $virus{$b} <=> $virus{$a};
}

push(@text_body, "Shows a count of each virus type:\n");

foreach $virus (sort hashValueDescendingNum (keys(%ip_addr))) {
	if ($ip_addr{$virus} >=  "$virus_count") {
		push(@text_body, "Count is $ip_addr{$virus} for $virus\n");
	}
}

if ($host_count) {
	push(@text_body, "\nShows uniques hosts with a virus count over $host_count:\n");
}


for $ip_addr (sort hashValueDescendingIp (keys(%virus))) {
#	add check to only show specific unique hosts -u address, address2
	if ($virus{"$ip_addr"} >= "$host_count") {


		push(@text_body, "\n$ip_addr sent the following virus's a total of $virus{$ip_addr} times: \n");
		
		foreach $Virus (sort hashValueDescendingVirus (keys (%{$email->{$ip_addr}{virus}}) )) {
			push(@text_body, "$Virus was transmitted $email->{$ip_addr}{virus}{$Virus} times.\n");
		}

		push(@text_body, "\n");

		if ($show_senders) {
#			modify to show -s address, address2 from command line			
#			if ($ip_addr eq "$ip_address") {		
				foreach $Sender (sort hashValueDescendingSender (keys( %{ $email->{$ip_addr}{sender} } ))) {
					push(@text_body, "Possibly spoofed sender address $Sender was seen $email->{$ip_addr}{sender}{$Sender} times.\n");
				}
#			}
		}

		if ($show_recipients) {
#			modify to show -r address, address2 from command line			
#			if ($ip_addr eq "$ip_address") {		
				foreach $Recipient (sort hashValueDescendingRecipient (keys( %{ $email->{$ip_addr}{recipient} } ))) {
					push(@text_body, "Recipient address $Recipient was seen $email->{$ip_addr}{recipient}{$Recipient} times.\n");
				}
#			}
		}
	}
}


	

#Email handling section

if ($send_email) {
	#Find hostname
	use Sys::Hostname;
	my $hostname = hostname();

use CGI qw(:standard);  
require MIME::Lite;
          
my $from_address = "$hostname <$hostname\@hillsboroughcounty.org>";

my $subject = "Subject: Analysis of logs for Virus's on $hostname\n";

# Create the initial text of the message
my $mime_msg = MIME::Lite->new(
   From => $from_address,
   To   => $mail_user,
   Subject => $subject,
   Type => 'TEXT',
   Data => "@text_body"
   )
  or die "Error creating MIME body: $!\n";

MIME::Lite->send('smtp', "$mail_server", Timeout=>60);

$mime_msg->send or die "Problems sending mail $!\n";

}

#Print out data
unless ($send_email) {
	for(my $counter=0 ; $counter < @text_body ; $counter++) {
		print "$text_body[$counter]" ;
	}
}

